# -*- coding: utf-8 -*-

"""Components to solve common issues with Navigator."""

from .branding_solvers import *
from .core import *
from .default_environment_solvers import *
from .ssl_certificate_solvers import *
from .trusted_servers_solvers import *



# -*- coding: utf-8 -*-

"""Constants for third-party applications."""

from __future__ import annotations

__all__ = ['EMPTY_PATCH']

import typing

from . import base


EMPTY_PATCH: typing.Final[base.AppPatch] = base.AppPatch()



# -*- coding: utf8 -*-

"""Basic event pool to process events in runtime-only."""

from __future__ import annotations

__all__ = ['BasicPool']

import collections.abc
import typing

from anaconda_navigator import config
from . import core


class BasicPool(core.Pool):
    """
    Basic event pool to process events in runtime-only.

    All events that are not submitted to the telemetry services will be lost.

    Service takes :code:`provide_analytics` preference into account when it processes events.
    """

    __slots__ = ('__content',)

    def __init__(self) -> None:
        """Initialize new instance of a :class:`~BasicPool`."""
        self.__content: typing.Final[dict[str, list[core.Event]]] = {}

    def pending(self, provider: str) -> list[core.Event]:
        """Retrieve all currently pending events for :code:`provider`."""
        result: list[core.Event] = self.__content.setdefault(provider, [])
        if not config.CONF.get('main', 'provide_analytics'):
            result.clear()
        return result

    def push(self, event: core.Event) -> None:
        """Push new :code:`event` to the pool."""
        if not config.CONF.get('main', 'provide_analytics'):
            return

        value: list[core.Event]
        for value in self.__content.values():
            value.append(event)

    def register(self, providers: collections.abc.Iterable[str]) -> None:
        """Register :code:`providers` in the pool for it to be aware of them."""
        provider: str
        for provider in providers:
            self.__content.setdefault(provider, [])



# -*- coding: utf-8 -*-

"""Base for Cloud-styled dialogs."""

from __future__ import annotations

__all__ = ['CloudDialogBase']

import typing

from qtpy import QtCore
from qtpy import QtGui
from qtpy import QtWidgets

from anaconda_navigator.static import images
from anaconda_navigator.utils import singletons


CLOSE_DIALOG_ICON: typing.Final[singletons.Singleton[QtGui.QIcon]] = singletons.SingleInstanceOf(
    QtGui.QIcon,
    images.CLOSE_DIALOG_ICON_PATH,
)


class CloudDialogFrame(QtWidgets.QFrame):  # pylint: disable=too-few-public-methods
    """Primary frame of :class:`~CloudDialogBase`."""


class CloudDialogTobBorder(QtWidgets.QFrame):  # pylint: disable=too-few-public-methods
    """Green border of :class:`~CloudDialogBase`."""


class CloudDialogTitleFrame(QtWidgets.QFrame):  # pylint: disable=too-few-public-methods
    """Container for title bar of :class:`~CloudDialogBase`."""


class CloudDialogTitleLabel(QtWidgets.QLabel):  # pylint: disable=too-few-public-methods
    """Title to show in :class:`~CloudDialogBase`."""


class CloudDialogCloseButton(QtWidgets.QPushButton):  # pylint: disable=too-few-public-methods
    """Close button in :class:`~CloudDialogBase`."""


class CloudDialogBodyFrame(QtWidgets.QFrame):  # pylint: disable=too-few-public-methods
    """Container for user content in :class:`~CloudDialogBase`."""


class CloudDialogBase(QtWidgets.QDialog):  # pylint: disable=too-many-instance-attributes
    """Common base for dialogs in Cloud style."""

    def __init__(self, parent: typing.Optional[QtCore.QObject] = None) -> None:
        """Initialize new :class:`~CloudDialogBase` instance."""
        super().__init__(parent)

        self.setMinimumHeight(480)
        self.setMinimumWidth(640)

        self._top_border: typing.Final[CloudDialogTobBorder] = CloudDialogTobBorder()

        self._title_label: typing.Final[CloudDialogTitleLabel] = CloudDialogTitleLabel()

        self._close_dialog_button: typing.Final[CloudDialogCloseButton] = CloudDialogCloseButton()
        self._close_dialog_button.setFixedSize(QtCore.QSize(24, 24))
        self._close_dialog_button.setIcon(CLOSE_DIALOG_ICON.instance)
        self._close_dialog_button.setIconSize(QtCore.QSize(24, 24))
        self._close_dialog_button.clicked.connect(lambda: self.reject())  # pylint: disable=unnecessary-lambda

        self._title_frame_layout: typing.Final[QtWidgets.QHBoxLayout] = QtWidgets.QHBoxLayout()
        self._title_frame_layout.setAlignment(QtCore.Qt.AlignCenter)
        self._title_frame_layout.setContentsMargins(8, 0, 8, 0)
        self._title_frame_layout.setSpacing(0)
        self._title_frame_layout.addWidget(self._title_label)
        self._title_frame_layout.addStretch(1)
        self._title_frame_layout.addWidget(self._close_dialog_button)

        self._title_frame: typing.Final[CloudDialogTitleFrame] = CloudDialogTitleFrame()
        self._title_frame.setLayout(self._title_frame_layout)
        self._title_frame.setContentsMargins(0, 0, 0, 0)

        self._body_frame: typing.Final[CloudDialogBodyFrame] = CloudDialogBodyFrame()

        self._dialog_frame_layout: typing.Final[QtWidgets.QVBoxLayout] = QtWidgets.QVBoxLayout()
        self._dialog_frame_layout.setContentsMargins(0, 0, 0, 0)
        self._dialog_frame_layout.setSpacing(0)
        self._dialog_frame_layout.addWidget(self._top_border)
        self._dialog_frame_layout.addWidget(self._title_frame)
        self._dialog_frame_layout.addWidget(self._body_frame)

        self._dialog_frame: typing.Final[CloudDialogFrame] = CloudDialogFrame()
        self._dialog_frame.setLayout(self._dialog_frame_layout)

        self._dialog_layout: typing.Final[QtWidgets.QVBoxLayout] = QtWidgets.QVBoxLayout()
        self._dialog_layout.setAlignment(QtCore.Qt.AlignTop)
        self._dialog_layout.setContentsMargins(0, 0, 0, 0)
        self._dialog_layout.setSpacing(10)
        self._dialog_layout.addWidget(self._dialog_frame)

        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)
        self.setSizeGripEnabled(False)
        self.setWindowFlags(QtCore.Qt.Dialog | QtCore.Qt.FramelessWindowHint)
        super().setLayout(self._dialog_layout)

    def layout(self) -> QtWidgets.QLayout:
        """Retrieve current dialog layout."""
        return self._body_frame.layout()

    def setWindowTitle(self, title: str) -> None:  # pylint: disable=invalid-name
        """Change title of the dialog window."""
        super().setWindowTitle(title)
        self._title_label.setText(title)

    def setLayout(self, layout: QtWidgets.QLayout) -> None:  # pylint: disable=invalid-name
        """Set content of the dialog."""
        self._body_frame.setLayout(layout)



# -*- coding: utf-8 -*-

"""Utilities to simplify configuration files management."""

from __future__ import annotations

__all__ = ['Updater', 'JsonUpdater', 'YamlUpdater']

import abc
import contextlib
import types
import typing
import json
import yaml


KEEP: typing.Any = object()


class Updater(contextlib.AbstractContextManager, metaclass=abc.ABCMeta):
    """
    Abstract helper that allows updating configuration files of general formats.

    It is used to hide general actions of reading and writing files with corresponding serialization.

    This class might also be used as a context manager. On successful exit - content of the configuration will be saved
    automatically.

    .. code-block:: python

        with JsonUpdater(path='/some/file') as configuration:
            configuration.content['some_property'] = 'new_value'

        # configuration file is saved automatically, unless an exception raised in main body

    :param path: Path to the configuration file.
    """

    __slots__ = ('__content', '__path')

    def __init__(self, path: str) -> None:
        """Initialize new :class:`~Updater` instance."""
        self.__path: str = path
        self.__content: typing.Any = self._read()

    @property
    def content(self) -> typing.Any:  # noqa: D401
        """Content of the configuration file."""
        return self.__content

    @content.setter
    def content(self, value: str) -> None:
        """Update `content` value."""
        self.__content = value

    @property
    def path(self) -> str:  # noqa: D401
        """Path to the configuration."""
        return self.__path

    @abc.abstractmethod
    def _read(self) -> typing.Any:
        """
        Read content of the configuration file.

        .. note::

            You can use value of the :attr:`~Updater.path` to locate the configuration file.

        .. warning::

            Do not use :attr:`~Updater.content`! This is the value that must be set by current function, and which must
            not rely on the previous configuration content.

        :return: Content of the configuration file.
        """

    @abc.abstractmethod
    def _write(self) -> None:
        """
        Write new configuration to the file.

        .. note::

            You can use value of the :attr:`~Updater.path` to locate the configuration file, and
            :attr:`~Updater.content` for the content of the configuration that should be saved.
        """

    def load(self) -> None:
        """Load configuration from the file."""
        self.__content = self._read()

    def save(self, path: str = KEEP) -> None:
        """
        Save configuration to the file.

        :param path: New path to save configuration to. This will also update value of the :attr:`~Updater.path` and
                     affect subsequent :meth:`~Updater.load` calls (unless there would be an error writing to file).

                     If not provided - current :attr:`~Updater.path` will be used.
        """
        path_backup: str = self.__path
        if path is not KEEP:
            self.__path = path
        try:
            self._write()
        except BaseException:
            self.__path = path_backup
            raise

    def __enter__(self) -> Updater:
        """Enter into the configuration updating context."""
        return self

    def __exit__(
            self,
            exc_type: typing.Optional[typing.Type[BaseException]],
            exc_val: typing.Optional[BaseException],
            exc_tb: typing.Optional[types.TracebackType],
    ) -> None:
        """
        Exit from the configuration updating context.

        If no exception raised - configuration would be saved.
        """
        if exc_val is None:
            self.save()


class EncodingAwareUpdater(Updater, metaclass=abc.ABCMeta):
    """
    Modified :class:`~Updater`, which also allows setting encoding to read configuration with.

    :param path: Path to the configuration file.
    :param encoding: Encoding to use for the configuration file.
    """

    __slots__ = ('__encoding',)

    def __init__(self, path: str, encoding: typing.Optional[str] = None) -> None:
        """Initialize new :class:`~EncodingAwareUpdater` instance."""
        self.__encoding: typing.Optional[str] = encoding
        super().__init__(path=path)

    @property
    def encoding(self) -> typing.Optional[str]:  # noqa: D401
        """Current encoding of the configuration file."""
        return self.__encoding

    def save(self, path: str = KEEP, encoding: typing.Optional[str] = KEEP) -> None:  # pylint: disable=arguments-differ
        """
        Save configuration to the file.

        :param path: New path to save configuration to. This will also update value of the
                     :attr:`~EncodingAwareUpdater.path` and affect subsequent :meth:`~Updater.load` calls (unless there
                     would be an error writing to file).

                     If not provided - current :attr:`~EncodingAwareUpdater.path` will be used.

        :param encoding: New encoding to save configuration file with. This also updates
                         :attr:`~EncodingAwareUpdater.encoding` in the way similar to `path` argument.
        """
        encoding_backup: typing.Optional[str] = self.__encoding
        if encoding is not KEEP:
            self.__encoding = encoding
        try:
            super().save(path=path)
        except BaseException:
            self.__encoding = encoding_backup
            raise


class JsonUpdater(EncodingAwareUpdater):
    """
    Configuration updater, that supports configuration files with JSON content.

    :param path: Path to the configuration file.
    :param encoding: Encoding to use for the configuration file.
    """

    __slots__ = ()

    def _read(self) -> typing.Any:
        """
        Read content of the configuration file.

        :return: Content of the configuration file.
        """
        stream: typing.TextIO
        with open(self.path, 'rt', encoding=self.encoding) as stream:
            return json.load(stream)

    def _write(self) -> None:
        """Write new configuration to the file."""
        stream: typing.TextIO
        with open(self.path, 'wt', encoding=self.encoding) as stream:
            json.dump(self.content, stream)


class YamlUpdater(EncodingAwareUpdater):
    """
    Configuration updater, that supports configuration files with YAML content.

    :param path: Path to the configuration file.
    :param encoding: Encoding to use for the configuration file.
    """

    __slots__ = ()

    def _read(self) -> typing.Any:
        """
        Read content of the configuration file.

        :return: Content of the configuration file.
        """
        stream: typing.TextIO
        with open(self.path, 'rt', encoding=self.encoding) as stream:
            return yaml.load(stream, yaml.FullLoader)  # nosec

    def _write(self) -> None:
        """Write new configuration to the file."""
        stream: typing.TextIO
        with open(self.path, 'wt', encoding=self.encoding) as stream:
            yaml.dump(self.content, stream)
